@@ -44,6 +44,8 @@
 #include "../declarations/classmethoddeclaration.h"
 #include "../declarations/classdeclaration.h"
 #include "../declarations/functiondeclaration.h"
+#include "../declarations/namespacedeclaration.h"
+#include "../declarations/namespacealiasdeclaration.h"
 #include "expressionvisitor.h"
 
 using namespace KTextEditor;
@@ -120,7 +122,7 @@ KDevelop::ReferencedTopDUContext DeclarationBuilder::build(const KDevelop::Index
     //Run DeclarationBuilder twice, to find uses of declarations that are
     //declared after the use. ($a = new Foo; class Foo {})
     {
-        PreDeclarationBuilder prebuilder(&m_types, &m_functions, &m_upcomingClassVariables, editor());
+        PreDeclarationBuilder prebuilder(&m_types, &m_functions, &m_namespaces, &m_upcomingClassVariables, editor());
         updateContext = prebuilder.build(url, node, updateContext, useSmart);
         m_actuallyRecompiling = prebuilder.didRecompile();
     }
@@ -133,12 +135,14 @@ KDevelop::ReferencedTopDUContext DeclarationBuilder::build(const KDevelop::Index
     } else if ( ICore::self() ) {
         m_reportErrors = ICore::self()->languageController()->completionSettings()->highlightSemanticProblems();
     }
+
     return ContextBuilderBase::build(url, node, updateContext, useSmart);
 }
 
 void DeclarationBuilder::startVisiting(AstNode* node)
 {
     setRecompiling(m_actuallyRecompiling);
+    setCompilingContexts(false);
     DeclarationBuilderBase::startVisiting(node);
 }
 
@@ -366,7 +370,7 @@ void DeclarationBuilder::visitClassExtends(ClassExtendsAst *node)
 
 void DeclarationBuilder::visitClassImplements(ClassImplementsAst *node)
 {
-    const KDevPG::ListNode<IdentifierAst*> *__it = node->implementsSequence->front(), *__end = __it;
+    const KDevPG::ListNode<NamespacedIdentifierAst*> *__it = node->implementsSequence->front(), *__end = __it;
     do {
         addBaseType(__it->element);
         __it = __it->next;
@@ -477,10 +481,9 @@ void DeclarationBuilder::declareClassMember(DUContext *parentCtx, AbstractType::
     closeInjectedContext(editor()->smart());
 }
 
-void DeclarationBuilder::visitClassConstantDeclaration(ClassConstantDeclarationAst *node)
+void DeclarationBuilder::visitConstantDeclaration(ConstantDeclarationAst *node)
 {
     if (m_reportErrors) {   // check for redeclarations
-        Q_ASSERT(currentContext()->type() == DUContext::Class);
         DUChainWriteLocker lock(DUChain::lock());
         foreach(Declaration * dec, currentContext()->findLocalDeclarations(identifierForNode(node->identifier).first(), startPos(node->identifier)))
         {
@@ -490,16 +493,14 @@ void DeclarationBuilder::visitClassConstantDeclaration(ClassConstantDeclarationA
             }
         }
     }
-    openDefinition<ClassMemberDeclaration>(node->identifier, node->identifier);
-    ClassMemberDeclaration* dec = dynamic_cast<ClassMemberDeclaration*>(currentDeclaration());
-    Q_ASSERT(dec);
+    ClassMemberDeclaration* dec = openDefinition<ClassMemberDeclaration>(node->identifier, node->identifier);
     {
         DUChainWriteLocker lock(DUChain::lock());
         dec->setAccessPolicy(Declaration::Public);
         dec->setStatic(true);
         dec->setKind(Declaration::Instance);
     }
-    DeclarationBuilderBase::visitClassConstantDeclaration(node);
+    DeclarationBuilderBase::visitConstantDeclaration(node);
     closeDeclaration();
     if ( m_reportErrors ) {
         // const class members may only be ints, floats, bools or strings
@@ -565,6 +566,48 @@ void DeclarationBuilder::visitFunctionDeclarationStatement(FunctionDeclarationSt
     closeType();
     closeDeclaration();
 }
+void DeclarationBuilder::visitClosure(ClosureAst* node)
+{
+    setComment(formatComment(node, editor()));
+    {
+        DUChainWriteLocker lock;
+        FunctionDeclaration *dec = openDefinition<FunctionDeclaration>(QualifiedIdentifier(),
+                                                                       editor()->findRange(node->startToken));
+        dec->setKind(Declaration::Type);
+        dec->clearDefaultParameters();
+    }
+
+    DeclarationBuilderBase::visitClosure(node);
+
+    closeDeclaration();
+}
+void DeclarationBuilder::visitLexicalVar(LexicalVarAst* node)
+{
+    DeclarationBuilderBase::visitLexicalVar(node);
+
+    QualifiedIdentifier id = identifierForNode(node->variable);
+    DUChainWriteLocker lock;
+    if ( recompiling() ) {
+        // sadly we can't use findLocalDeclarations() here, since it un-aliases declarations
+        foreach ( Declaration* dec, currentContext()->localDeclarations() ) {
+            if ( dynamic_cast<AliasDeclaration*>(dec) && dec->identifier() == id.first() ) {
+                // don't redeclare but reuse the existing declaration
+                encounter(dec);
+                return;
+            }
+        }
+    }
+
+    // no existing declaration found, create one
+    foreach(Declaration* aliasedDeclaration, currentContext()->findDeclarations(id)) {
+        if (aliasedDeclaration->kind() == Declaration::Instance) {
+            AliasDeclaration* dec = openDefinition<AliasDeclaration>(id, editor()->findRange(node->variable));
+            dec->setAliasedDeclaration(aliasedDeclaration);
+            closeDeclaration();
+            break;
+        }
+    }
+}
 
 bool DeclarationBuilder::isGlobalRedeclaration(const QualifiedIdentifier &identifier, AstNode* node,
         DeclarationType type)
@@ -608,13 +651,13 @@ void DeclarationBuilder::reportRedeclarationError(Declaration* declaration, AstN
         );
     }
 }
-void DeclarationBuilder::visitTopStatement(TopStatementAst* node)
+void DeclarationBuilder::visitOuterTopStatement(OuterTopStatementAst* node)
 {
     //docblock of an AssignmentExpression
     setComment(formatComment(node, editor()));
     m_lastTopStatementComment = editor()->parseSession()->docComment(node->startToken);
 
-    DeclarationBuilderBase::visitTopStatement(node);
+    DeclarationBuilderBase::visitOuterTopStatement(node);
 }
 
 void DeclarationBuilder::visitAssignmentExpression(AssignmentExpressionAst* node)
@@ -659,6 +702,8 @@ void DeclarationBuilder::declareVariable(DUContext* parentCtx, AbstractType::Ptr
                                             AstNode* node)
 {
     // we must not re-assign $this in a class context
+    /// Qualified identifier for 'this'
+    static const QualifiedIdentifier thisQId("this");
     if ( identifier == thisQId
             && currentContext()->parentContext()
             && currentContext()->parentContext()->type() == DUContext::Class ) {
@@ -746,6 +791,8 @@ void DeclarationBuilder::declareVariable(DUContext* parentCtx, AbstractType::Ptr
 }
 
 DUContext* getClassContext(const QualifiedIdentifier &identifier, DUContext* currentCtx) {
+    /// Qualified identifier for 'this'
+    static const QualifiedIdentifier thisQId("this");
     if ( identifier == thisQId ) {
         if ( currentCtx->parentContext() && currentCtx->parentContext()->type() == DUContext::Class ) {
             return currentCtx->parentContext();
@@ -795,6 +842,7 @@ void DeclarationBuilder::visitAssignmentExpressionEqual(AssignmentExpressionEqua
 
 void DeclarationBuilder::visitFunctionCall(FunctionCallAst* node)
 {
+    QualifiedIdentifier id;
     {
         FunctionType::Ptr oldFunction = m_currentFunctionType;
 
@@ -802,7 +850,8 @@ void DeclarationBuilder::visitFunctionCall(FunctionCallAst* node)
         if ( node->stringFunctionName ) {
             dec = findDeclarationImport(FunctionDeclarationType, node->stringFunctionName);
         } else if ( node->stringFunctionNameOrClass ) {
-            dec = findDeclarationImport(FunctionDeclarationType, node->stringFunctionNameOrClass);
+            id = identifierForNamespace(node->stringFunctionNameOrClass, editor());
+            dec = findDeclarationImport(FunctionDeclarationType, id, node->stringFunctionNameOrClass);
         } else {
             ///TODO: node->varFunctionName
         }
@@ -819,7 +868,7 @@ void DeclarationBuilder::visitFunctionCall(FunctionCallAst* node)
     }
 
     if (node->stringFunctionNameOrClass && !node->stringFunctionName && !node->varFunctionName) {
-        if (stringForNode(node->stringFunctionNameOrClass).compare("define", Qt::CaseInsensitive) == 0
+        if (id.toString(true) == "define"
                 && node->stringParameterList && node->stringParameterList->parametersSequence
                 && node->stringParameterList->parametersSequence->count() > 0) {
             //constant, defined through define-function
@@ -832,7 +881,13 @@ void DeclarationBuilder::visitFunctionCall(FunctionCallAst* node)
                 SimpleRange newRange = editorFindRange(scalar, scalar);
                 DUChainWriteLocker lock(DUChain::lock());
                 LockedSmartInterface iface = editor()->smart();
-                injectContext(iface, currentContext()->topContext()); //constants are always global
+                // find fitting context to put define in,
+                // pick first namespace or global context otherwise
+                DUContext* ctx = currentContext();
+                while (ctx->type() != DUContext::Namespace && ctx->parentContext()) {
+                    ctx = ctx->parentContext();
+                }
+                injectContext(iface, ctx); //constants are always global
                 QualifiedIdentifier identifier(constant);
                 isGlobalRedeclaration(identifier, scalar, ConstantDeclarationType);
                 openDefinition<Declaration>(identifier, newRange);
@@ -976,8 +1031,6 @@ void DeclarationBuilder::visitStatement(StatementAst* node)
         openDefinition<VariableDeclaration>(identifierForNode(node->foreachVariable->variable), newRange);
         currentDeclaration()->setKind(Declaration::Instance);
         closeDeclaration();
-
-        clearLastType();
     }
 
     if (node->foreachVarAsVar) {
@@ -1077,6 +1130,44 @@ void DeclarationBuilder::visitUnaryExpression(UnaryExpressionAst* node)
     }
 }
 
+void DeclarationBuilder::openNamespace(NamespaceDeclarationStatementAst* parent, IdentifierAst* node, const IdentifierPair& identifier, const Range& range)
+{
+    NamespaceDeclaration* dec = m_namespaces.value(node->string, 0);
+    Q_ASSERT(dec);
+    DeclarationBuilderBase::setEncountered(dec);
+    openDeclarationInternal(dec);
+
+    DeclarationBuilderBase::openNamespace(parent, node, identifier, range);
+}
+
+void DeclarationBuilder::closeNamespace(NamespaceDeclarationStatementAst* parent, IdentifierAst* node, const IdentifierPair& identifier)
+{
+    DeclarationBuilderBase::closeNamespace(parent, node, identifier);
+    closeDeclaration();
+}
+
+void DeclarationBuilder::visitUseNamespace(UseNamespaceAst* node)
+{
+    if ( !node->aliasIdentifier && node->identifier->namespaceNameSequence->count() == 1 ) {
+        reportError(i18n("The use statement with non-compound name '%1' has no effect.",
+                         identifierForNode(node->identifier->namespaceNameSequence->front()->element).toString()),
+                    node->identifier, KDevelop::ProblemData::Warning);
+        return;
+    }
+    IdentifierAst* idNode = node->aliasIdentifier ? node->aliasIdentifier : node->identifier->namespaceNameSequence->back()->element;
+    IdentifierPair id = identifierPairForNode(idNode);
+    SimpleRange range = editor()->findRange(idNode);
+    DUChainWriteLocker lock;
+    NamespaceAliasDeclaration* decl = openDefinition<NamespaceAliasDeclaration>(id.second, range);
+    {
+        ///TODO: case insensitive!
+        decl->setImportIdentifier( identifierForNamespace(node->identifier, editor()) );
+        decl->setPrettyName( id.first );
+        decl->setKind(Declaration::NamespaceAlias);
+    }
+    closeDeclaration();
+}
+
 void DeclarationBuilder::updateCurrentType()
 {
     DUChainWriteLocker lock(DUChain::lock());
