@@ -316,6 +316,7 @@ void TestUses::constant()
 
 void TestUses::classConstant()
 {
+    {
     //                 0         1         2         3         4         5         6         7
     //                 01234567890123456789012345678901234567890123456789012345678901234567890123456789
     QByteArray method("<? class A { const FOO = 'abc'; } echo A::FOO;");
@@ -325,6 +326,27 @@ void TestUses::classConstant()
 
     compareUses(top->localDeclarations().first(), SimpleRange(0, 39, 0, 40));
     compareUses(top->childContexts().first()->localDeclarations().first(), SimpleRange(0, 42, 0, 45));
+    }
+    {
+    // bug: https://bugs.kde.org/show_bug.cgi?id=241597
+
+    //                 0         1         2         3         4         5         6         7
+    //                 01234567890123456789012345678901234567890123456789012345678901234567890123456789
+    QByteArray method("<? class A { const FOO = 'abc'; }\n"
+                      "class B extends A { function foo() { self::FOO; } }\n"
+                      "A::FOO;\n"
+                      "B::FOO;\n");
+    TopDUContext* top = parse(method, DumpAll);
+    DUChainReleaser releaseTop(top);
+    DUChainWriteLocker lock;
+
+    Declaration* dec = top->childContexts().first()->localDeclarations().first();
+    QVERIFY(dec->abstractType()->modifiers() & AbstractType::ConstModifier);
+    QCOMPARE(dec->qualifiedIdentifier().toString(), QString("a::FOO"));
+    compareUses(dec, QList<SimpleRange>() << SimpleRange(1, 43, 1, 46)
+                                          << SimpleRange(2, 3, 2, 6)
+                                          << SimpleRange(3, 3, 3, 6));
+    }
 }
 
 void TestUses::classParent()
@@ -828,7 +850,170 @@ void TestUses::functionArguments()
     compareUses(d, SimpleRange(0, 35, 0, 37));
 }
 
+void TestUses::namespaces()
+{
+    //                         0         1         2         3         4         5         6         7
+    //                         01234567890123456789012345678901234567890123456789012345678901234567890123456789
+    TopDUContext* top = parse("<?php\n"
+                              "namespace Foo\\Bar {\n"
+                              "const MyConst = 1;\n"
+                              "function MyFunc(){}\n"
+                              "class MyClass{ const ClassConst = 2; }\n"
+                              "interface MyInterface{}\n"
+                              "}\n"
+                              "namespace {\n"
+                              "\\Foo\\Bar\\MyConst;\n"
+                              "\\Foo\\Bar\\MyClass::ClassConst;\n"
+                              "\\Foo\\Bar\\MyFunc();\n"
+                              "new \\Foo\\Bar\\MyClass;\n"
+                              "function Func(\\Foo\\Bar\\MyClass $a){}\n"
+                              "class a extends \\Foo\\Bar\\MyClass implements \\Foo\\Bar\\MyInterface {}\n"
+                              "}\n", DumpAll);
+    QVERIFY(top);
+    DUChainReleaser releaseTop(top);
+    DUChainWriteLocker lock;
+
+    Declaration* dec;
+
+    dec = top->findDeclarations(QualifiedIdentifier("foo")).first();
+    QCOMPARE(dec->kind(), Declaration::Namespace);
+    compareUses(dec, QList<SimpleRange>() << SimpleRange(8, 1, 8, 4)
+                                          << SimpleRange(9, 1, 9, 4)
+                                          << SimpleRange(10, 1, 10, 4)
+                                          << SimpleRange(11, 5, 11, 8)
+                                          << SimpleRange(12, 15, 12, 18)
+                                          << SimpleRange(13, 17, 13, 20)
+                                          << SimpleRange(13, 45, 13, 48));
+
+    dec = top->findDeclarations(QualifiedIdentifier("foo::bar")).first();
+    QCOMPARE(dec->kind(), Declaration::Namespace);
+    QVERIFY(dec->internalContext());
+    compareUses(dec, QList<SimpleRange>() << SimpleRange(8, 5, 8, 8)
+                                          << SimpleRange(9, 5, 9, 8)
+                                          << SimpleRange(10, 5, 10, 8)
+                                          << SimpleRange(11, 9, 11, 12)
+                                          << SimpleRange(12, 19, 12, 22)
+                                          << SimpleRange(13, 21, 13, 24)
+                                          << SimpleRange(13, 49, 13, 52));
+    QCOMPARE(dec->internalContext()->localDeclarations().size(), 4);
+    foreach(Declaration* d, dec->internalContext()->localDeclarations()) {
+        kDebug() << d->toString() << d->qualifiedIdentifier();
+    }
+
+    dec = top->findDeclarations(QualifiedIdentifier("foo::bar::MyConst")).first();
+    compareUses(dec, SimpleRange(8, 9, 8, 16));
+
+    dec = top->findDeclarations(QualifiedIdentifier("foo::bar::myclass")).first();
+    QVERIFY(dynamic_cast<ClassDeclaration*>(dec));
+    compareUses(dec, QList<SimpleRange>() << SimpleRange(9, 9, 9, 16)
+                                          << SimpleRange(11, 13, 11, 20)
+                                          << SimpleRange(12, 23, 12, 30)
+                                          << SimpleRange(13, 25, 13, 32)
+               );
+    dec = top->findDeclarations(QualifiedIdentifier("foo::bar::myinterface")).first();
+    QVERIFY(dynamic_cast<ClassDeclaration*>(dec));
+    compareUses(dec, SimpleRange(13, 53, 13, 64) );
+
+    dec = top->findDeclarations(QualifiedIdentifier("foo::bar::myclass::ClassConst")).first();
+    compareUses(dec, SimpleRange(9, 18, 9, 28));
+
+    dec = top->findDeclarations(QualifiedIdentifier("foo::bar::myfunc")).first();
+    compareUses(dec, SimpleRange(10, 9, 10, 15));
 }
 
+void TestUses::useNamespace()
+{
+    //                         0         1         2         3         4         5         6         7
+    //                         01234567890123456789012345678901234567890123456789012345678901234567890123456789
+    TopDUContext* top = parse("<?php\n"
+                              "namespace Foo\\Bar {class A{} function B(){} const C = 1;}\n"
+                              "namespace VeryLong {class A{} function B(){} const C = 1;}\n"
+                              "namespace {\n"
+                              "use Foo\\Bar, VeryLong as Short;\n"
+                              "new Bar\\A; Bar\\B(); Bar\\C;\n"
+                              "new Short\\A; Short\\B(); Short\\C;\n"
+                              "}\n", DumpNone);
+    QVERIFY(top);
+    DUChainReleaser releaseTop(top);
+    DUChainWriteLocker lock;
+
+    Declaration* dec;
+    dec = top->findDeclarations(QualifiedIdentifier("foo")).first();
+    QCOMPARE(dec->kind(), Declaration::Namespace);
+    compareUses(dec, SimpleRange(4, 4, 4, 7));
+
+    dec = top->findDeclarations(QualifiedIdentifier("foo::bar")).first();
+    QCOMPARE(dec->kind(), Declaration::Namespace);
+    compareUses(dec, SimpleRange(4, 8, 4, 11));
+
+    dec = top->findDeclarations(QualifiedIdentifier("verylong")).first();
+    QCOMPARE(dec->kind(), Declaration::Namespace);
+    compareUses(dec, SimpleRange(4, 13, 4, 21));
+
+    dec = top->findDeclarations(QualifiedIdentifier("bar")).first();
+    QCOMPARE(dec->kind(), Declaration::NamespaceAlias);
+    compareUses(dec, QList<SimpleRange>() << SimpleRange(5, 4, 5, 7)
+                                          << SimpleRange(5, 11, 5, 14)
+                                          << SimpleRange(5, 20, 5, 23) );
+
+    dec = top->findDeclarations(QualifiedIdentifier("short")).first();
+    QCOMPARE(dec->kind(), Declaration::NamespaceAlias);
+    compareUses(dec, QList<SimpleRange>() << SimpleRange(6, 4, 6, 9)
+                                          << SimpleRange(6, 13, 6, 18)
+                                          << SimpleRange(6, 24, 6, 29) );
+
+    dec = top->findDeclarations(QualifiedIdentifier("foo::bar::a")).first();
+    compareUses(dec, SimpleRange(5, 8, 5, 9));
+
+    dec = top->findDeclarations(QualifiedIdentifier("foo::bar::b")).first();
+    compareUses(dec, SimpleRange(5, 15, 5, 16));
+
+    dec = top->findDeclarations(QualifiedIdentifier("foo::bar::C")).first();
+    compareUses(dec, SimpleRange(5, 24, 5, 25));
+
+    dec = top->findDeclarations(QualifiedIdentifier("verylong::a")).first();
+    compareUses(dec, SimpleRange(6, 10, 6, 11));
+
+    dec = top->findDeclarations(QualifiedIdentifier("verylong::b")).first();
+    compareUses(dec, SimpleRange(6, 19, 6, 20));
+
+    dec = top->findDeclarations(QualifiedIdentifier("verylong::C")).first();
+    compareUses(dec, SimpleRange(6, 30, 6, 31));
+}
+
+void TestUses::lateStatic()
+{
+    //                         0         1         2         3         4         5         6         7
+    //                         01234567890123456789012345678901234567890123456789012345678901234567890123456789
+    TopDUContext* top = parse("<?php class a { function b() { static::b(); } }", DumpAll);
+    QVERIFY(top);
+    DUChainReleaser releaseTop(top);
+    DUChainWriteLocker lock;
+
+    compareUses(top->childContexts().first()->localDeclarations().first(), SimpleRange(0, 39, 0, 40));
+}
+
+void TestUses::closures()
+{
+    //                         0         1         2         3         4         5         6         7
+    //                         01234567890123456789012345678901234567890123456789012345678901234567890123456789
+    TopDUContext* top = parse("<?php $a = 1; $b = 2;\n"
+                              "$l = function($b) use ($a) { return $a - $b; };\n", DumpNone);
+    QVERIFY(top);
+    DUChainReleaser releaseTop(top);
+    DUChainWriteLocker lock;
+
+    QCOMPARE(top->localDeclarations().count(), 4);
+
+    Declaration* a = top->localDeclarations().at(0);
+    QCOMPARE(a->identifier().toString(), QString("a"));
+    compareUses(a, QList<SimpleRange>() << SimpleRange(1, 23, 1, 25) << SimpleRange(1, 36, 1, 38));
+
+    Declaration* b = top->localDeclarations().at(1);
+    QCOMPARE(b->identifier().toString(), QString("b"));
+    QVERIFY(b->uses().isEmpty());
+}
+
+}
 
 #include "uses.moc"
